/***************************************************************************************
ガスセンサの値を読み取ってキャラクタ液晶に表示する
DFROBOT LCD Keypad Shield用
メタンガスセンサ	MQ-4
一酸化炭素センサ	MQ-7

                                                       Copyright (c) 2013 Wataru KUNINO
***************************************************************************************/

#include <LiquidCrystal.h>
#include "pitches.h"

#define		PIN_LED			13					// Digital 13に動作確認用LEDを接続
#define		PIN_BUZZER		2					// Digital 2にブザーを接続

#define		PIN_CO			0					// Analog 0に一酸化炭素ガスセンサ
#define		PIN_CH4			1					// Analog 1にメタンガスセンサ
#define		PIN_CO_VSS		11					// Digital 11にCOガスのVSS側
#define		PIN_CO_PWM		75					// COガスセンサVSS側のPWM値

#define CO_Ro			2						// COガス100ppm時のセンサ抵抗値[kΩ]
#define CH4_Ro			10						// メタン1000ppm時のセンサ抵抗値[kΩ]

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
byte trig=0;									// COガスセンサの測定状態
byte buzzer=0x00;								// ブザー状態
unsigned long time;								// 現在の時間
unsigned long time_next;						// 次回の測定状態変更時間
boolean time_ovf = false;						// 桁上がり状況のフラグ
int val;										// CH4,COガスの読み取り値
float ch4,co;									// CH4,COガスの測定値

void setup(){
	pinMode(PIN_CO_VSS,OUTPUT);					// COガス用の電源ポートを出力に設定
	digitalWrite(PIN_CO_VSS,LOW);				// 出力を 0V に設定
	lcd.begin(16, 2);							// LCDのサイズを16文字×2桁に設定
	lcd.clear();								// 画面消去
	lcd.print("Gas Sensors");					// 文字を表示
	time_next = millis() + 1000;				// 1秒後にCOガス測定状態を変更する
}

void loop(){
	time= millis();
	digitalWrite(PIN_LED,HIGH);
	if( time_ovf && time < 1000 ) time_ovf=false;	// timerリセット(50日に一度)処理
	if( time > time_next && !time_ovf){				// 状態変更時刻になった時
		switch(trig){
			case 0:									// 測定の実行
				val = analogRead(PIN_CO);  			// 一酸化炭素ガスセンサの値を取得
				co = pow(10,2.0-1.7*log10(10*(1023/(float)val-1)/CO_Ro));
				digitalWrite(PIN_CO_VSS,HIGH);		// COガスの電源を 5V に設定
				time_next += 60000;					// 60秒後に設定
				if( time_next < 60000 ) time_ovf = true;
				trig++;
				break;
			case 2:									// 測定待ち
				analogWrite(PIN_CO_VSS,PIN_CO_PWM);	// COガスの電源を 1.4V に設定
				time_next += 90000;					// 90秒後に設定
				if( time_next < 90000 ) time_ovf = true;
				trig=0;
				break;
		}
	}
	val = analogRead(PIN_CH4);						// メタンガスセンサ読み取り
	ch4 = pow(10,3.0-2.7*log10(22*(1023/(float)val-1)/CH4_Ro));

	if(trig==1){
	//	tone(PIN_BUZZER,NOTE_A7,100); delay(200);
	//	tone(PIN_BUZZER,NOTE_A7,100);
		trig++;
		buzzer=0x00;
	}
	
	/* 表示 */
	lcd.clear();									// 液晶の画面消去
	lcd.print("CO =");
	if( co < 10 ) lcd.print( co , 2);				// 変数coの値を表示
	else lcd.print( co , 0);
	lcd.print("ppm");								// 変数ch4の値を表示
	lcd.setCursor(12, 0);							// 液晶の後方へ移動
	if( trig == 0 ) lcd.print("COOL");
	else lcd.print("HEAT");
	lcd.setCursor(0, 1);							// 液晶の２行目に移動
	lcd.print("CH4=");								// 液晶に文字を表示
	if( ch4 < 10 ) lcd.print( ch4 , 2);				// 変数ch4の値を表示
	else lcd.print( ch4 , 0);
	lcd.print("ppm");								// 変数ch4の値を表示
	lcd.setCursor(12, 1);							// 液晶の２行目の後方へ移動
	lcd.print( (time/1000)%3600 );
	digitalWrite(PIN_LED,LOW);
	
	/* ブザー */
	if(buzzer){
		if(buzzer&0x01){
			if(buzzer&0x10)	tone(PIN_BUZZER,NOTE_B7,100);
			else			tone(PIN_BUZZER,NOTE_B7,300);
			buzzer++;
		}else{
			if(buzzer&0x10)	tone(PIN_BUZZER,NOTE_G7,100);
			else			tone(PIN_BUZZER,NOTE_G7,300);
			buzzer--;
		}
	}else if(co >  100)	buzzer=0x01;					// COが優先
	else if(co  >   50)	tone(PIN_BUZZER,NOTE_A7,300);
	else if(ch4 > 1000)	buzzer=0x11;
	else if(ch4 >  100)	tone(PIN_BUZZER,NOTE_A7,100);
	
	/* 待ち時間 */
	delay(500);
}

/*
参考文献
	TECHNICAL DATA MQ-4 GAS SENSOR HANWAEI ELECTRONICS
	TECHNICAL DATA MQ-7 GAS SENSOR HANWAEI ELECTRONICS
*/
